/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.peer.curses;

/**
 * A <b>curses</b> color. This is the object that will be stored
 * in {@link org.nongnu.savannah.jcurzez.Color#realColor realColor}
 * when using the <b>curses</b> implementation.
 * <p>
 * Each color is defined by :
 * <ul>
 * <li>an index</li>
 * <li>a red component</li>
 * <li>a green component</li>
 * <li>a blue component</li>
 * </ul>
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public final class CursesColor {

    /**
     * Creates a new color with specified intensified values, using RGB
     * color system.
     *
     * @param index the index of the color.
     * @param red red intensity, in the range 0-1000.
     * @param green green intensity, in the range 0-1000.
     * @param blue blue intensity, in the range 0-1000.
     */
    public CursesColor(short index, short red, short green, short blue) {
	this.index = index;
	this.red = red;
	this.green = green;
	this.blue = blue;
    }

    /**
     * Returns a string representation of this <code>CursesColor</code>,
     * i&#46;e&#46; the intensity of red, green and blue.
     *
     * @return the color in the stringified form "[red=r,green=g,blue=b]".
     */
    public String toString() {
	return "[red=" + red +  ", green=" + green + ", blue=" + blue + "]";
    }

    /**
     * Returns the index of the color.
     *
     * @return the index of this <code>CursesColor</code>.
     */
    public short getIndex() {
	return index;
    }

    /**
     * Returns the red component of this <code>CursesColor</code>.
     *
     * @return the red component.
     */
    public short getRed() {
	return red;
    }

    /**
     * Returns the green component of this <code>CursesColor</code>.
     *
     * @return the green component.
     */
    public short getGreen() {
	return green;
    }

    /**
     * Returns the blue component of this <code>CursesColor</code>.
     *
     * @return the blue component.
     */
    public short getBlue() {
	return blue;
    }

    /**
     * Index of the color for <b>curses</b>.
     */
    short index;

    /**
     * Red component of this <code>CursesColor</code>.
     */
    short red;

    /**
     * Green component of this <code>CursesColor</code>.
     */
    short green;

    /**
     * Blue component of this <code>CursesColor</code>.
     */
    short blue;
}
