/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.test;

import org.nongnu.savannah.jcurzez.*;

/**
 * This class is a generic test for peer implementations.
 *
 * @version 1.0.0
 * @author Thomas Girard
 */
public class Test {
    public Test(AbstractWindow aw, long pause) {
	try {
	    //wrapping
	    aw.clear();
	    aw.refresh();
	    aw.printString("123456789012345678901234567890123456789" +
			   "01234567890123456789012345678901234567890" +
			   "1234567890\n");
	    aw.printString("1234567890123456789012345678901234567890\n");
	    aw.refresh();
	    Thread.sleep(pause);

	    //moving
	    aw.getCursor().setY(1);
	    aw.refresh();
	    Thread.sleep(pause);

	    //line insertion
	    aw.insertLine();
	    aw.refresh();
	    Thread.sleep(pause);

	    //line deletion
	    aw.deleteLine();
	    aw.refresh();
	    Thread.sleep(pause);

	    //Return to origin.
	    aw.getCursor().setXY(0,0);
	    aw.refresh();
	    Thread.sleep(pause);

	    //character deletion
	    aw.deleteChar();
	    aw.refresh();
	    Thread.sleep(pause);

	    //character insertion
	    aw.insertChar('1');
	    aw.refresh();
	    Thread.sleep(pause);

	    //string insertion
	    //aw.insertChar('!');
	    aw.insertString("Whatever");
	    aw.refresh();
	    Thread.sleep(pause);

	    //clear to end of line
	    aw.clearToEndOfLine();
	    aw.refresh();
	    Thread.sleep(pause);

	    //clear to bottom
	    aw.clearToBottom();
	    aw.refresh();
	    Thread.sleep(pause);

	    //writing in the bottom right corner
	    aw.getCursor().setXY(aw.getRectangle().getWidth() -1,
				 aw.getRectangle().getHeight() -1);
	    aw.insertChar((int) 'O');
	    aw.refresh();
	    Thread.sleep(pause);

	    //clear
	    aw.getCursor().setXY(0, 0);

	    Pen pen = aw.getRectangle().getFillingPen();
	    pen.setBackgroundColor(BackgroundColor.BLACK);
	    pen.setForegroundColor(ForegroundColor.WHITE);
	    pen.setAttribute(Attribute.NORMAL);
	    aw.clear();
	    aw.refresh();
	    Thread.sleep(pause);
	} catch(InterruptedException ie) {
	    System.out.println("Interrupted");
	} catch(Throwable t) {
	    System.out.println(t.getMessage());
	}
    }
}
