/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.test;

import org.nongnu.savannah.jcurzez.*;
import atomicsets.OrderedLock;
import java.util.Date;
import java.text.SimpleDateFormat;


/**
 * A little test program to test <b>Jcurzez</b> thread safety.
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public class ThreadSafetyAreas {
    /**
     * The thread group that owns all the threads this
     * test will create.
     */
    private static final ThreadGroup threads;


    /**
     * We set a shutdown hook to kill all threads upon termination.
     */
    static {
	threads = new ThreadGroup("clocks");
	/*Runtime.getRuntime().addShutdownHook(new Thread() {
		public void run() {
		    synchronized(threads) {
			//Kill'em all
			threads.interrupt();
		    }
		}
		});*/
    }

    /** To display the date like I want. */
    private static final SimpleDateFormat df = new SimpleDateFormat("HH:mm:ss");

    /** The screen. */
    private static final Screen screen = Screen.getScreen();

    /** Width of the screen. */
    private static final int    width  = screen.getRectangle().getWidth();

    /** Height of the screen. */
    private static final int    height = screen.getRectangle().getHeight();

    /**
     * Writes time at position <code>x</code>, <code>y</code>
     */
    private static class ClockAreaThread extends Thread {
	private long t1;
	private final Date d;
	private final Area myArea;

	private ClockAreaThread(int x, int y) {
	    super(threads, "Clock at x: " + x + ", y: " + y);
	    this.t1 = System.currentTimeMillis();
	    this.d = new Date(t1);
	    this.myArea = screen.constructChildArea(new Rectangle(x,y,x+8,y+1, new OrderedLock()), true, false);  
	    //new Area(screen,new Rectangle(x,y,x+8,y+1),true,false,true);
	    setDaemon(true);
	}

	public void run() {
		
	    while(!isInterrupted()) {
		t1 = System.currentTimeMillis();
		d.setTime(t1);
	
		myArea.getCursor().setXY(0,0);
		myArea.printString(df.format(d));
	    myArea.refresh();
	    
		try {
		    sleep(1000);
		} catch(InterruptedException ie) {
		    return;
		}
	    }
	}
    }


    /**
     * Test thread safety by having multiple threads write on the screen.
     *
     * @exception InterruptedException raised if the sleep is interrupted.
     */
    public static void main(String[] args) throws InterruptedException {
	screen.clear();
	screen.getCursor().setCursorVisibility(0);
	screen.refresh();

	
	for (int i = 0; i + 8 < width; i += 9) {
	    for(int j = 0; j < height; j ++) {
	    	new ClockAreaThread(i, j).start();
	    }
	}
	
//	ClockAreaThread[][] arr = new ClockAreaThread[width][height];
//	
//	for (int i = 0; i + 8 < width; i += 9) {
//	    for(int j = 0; j < height; j ++) {
//		arr[i][j] = new ClockAreaThread(i, j);
//	    }
//	}
//	
//	for (int i = 0; i + 8 < width; i += 9) {
//	    for(int j = 0; j < height; j ++) {
//		arr[i][j].start();
//	    }
//	}

	//Sleep for 10 seconds. Since threads are daemons,
	//they will be killed after this sleep.
	Thread.sleep(10000);
    }
}
