/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.test;

import atomicsets.OrderedLock;
import org.nongnu.savannah.jcurzez.*;
import org.nongnu.savannah.jcurzez.frame.*;

/**
 * Tests the {@link jcurzez.Window Window} object.
 *
 * @author Thomas Girard
 * @version 1.0.0
 */

// The green window should be printing its messages quickly and evenly without
// getting held up behind the red window who takes a long time to write things
// atomically (see AbstractWindow.stressConcurrency).

public class ThreadedTestConcurrency {
	
	public static String frameClassName;
	
    /**
     * Runs the {@link org.nongnu.savannah.jcurzez.test.Test} program using the
     * <code>Window</code> object.
     * <br>
     * One can specify the frame to use with
     * <code>--frame=<i>frameclassname</i></code>.
     */
    public static void main(String[] args) throws Exception {
		if (args.length > 0) {
		    if (args[0].startsWith("--frame=")) {
		    	frameClassName = args[0].substring(8);
		    } else {
		    	throw new
			    	IllegalArgumentException("The only parameter that can be " +
						     "used is --frame=_classname_");
		    }
		} else {
		    frameClassName = "org.nongnu.savannah.jcurzez.frame.SingleFrame";
		}
		
		Screen.getScreen().touch();
		Screen.getScreen().refresh();
	
		new Thread(){
			public void run(){
				Window win = /*new*/ Window.getWindow(new Rectangle(5,5,30,20, new OrderedLock()));
				win.getRectangle().getFillingPen().setBackgroundColor(BackgroundColor.RED);
				win.getRectangle().getDrawingPen().setBackgroundColor(BackgroundColor.RED);
				win.stressConcurrency = true;
				win.clear();
				win.refresh();
				
				for (int i = 0 ; i < 5; i++){
					win.printString(i + ")  This is a long message that when printed in a window " +
							"that is artificially stressing concurrency will cause some lock to be held " +
							"for a long time.  Hopefully, other windows can proceed while the lock is held.\n\n");
					win.refresh();
					try{Thread.sleep(500);}catch(Exception e){}
				}
			}
		}.start();
		
		
		Thread agile = new Thread(){
			public void run(){
				Window win = /*new*/ Window.getWindow(new Rectangle(45,5,75,20, new OrderedLock()));
				win.getRectangle().getFillingPen().setBackgroundColor(BackgroundColor.GREEN);
				win.getRectangle().getDrawingPen().setBackgroundColor(BackgroundColor.GREEN);
				win.stressConcurrency = false;
				win.clear();
				win.refresh();
				
				int i = 0;
				while (true){
					win.printString("Fast message " + i + ".\n");
					win.refresh();
					try{Thread.sleep(250);}catch(Exception e){}
					i++;
				}
			}
		};
		
		agile.setDaemon(true);
		
		agile.start();
		
    }
}