/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.test;

import org.nongnu.savannah.jcurzez.*;
import org.nongnu.savannah.jcurzez.frame.*;

import atomicsets.OrderedLock;
import java.text.NumberFormat;

/**
 * Tests the {@link jcurzez.Window Window} object.
 *
 * @author Thomas Girard
 * @version 1.0.0
 */

// The green window should be printing its messages quickly and evenly without
// getting held up behind the red window who takes a long time to write things
// atomically (see AbstractWindow.stressConcurrency).

public class ThreadedTestConcurrency3 {
	
	public static String frameClassName;
	
	static int num=0;
	
	static int interval = 1;
	
    /**
     * Runs the {@link org.nongnu.savannah.jcurzez.test.Test} program using the
     * <code>Window</code> object.
     * <br>
     * One can specify the frame to use with
     * <code>--frame=<i>frameclassname</i></code>.
     */
    public static void main(String[] args) throws Exception {
		if (args.length > 0) {
		    if (args[0].startsWith("--frame=")) {
		    	frameClassName = args[0].substring(8);
		    } else if (args[0].startsWith("--interval=")){
		    	interval = Integer.parseInt(args[0].substring(11));
		    } else {
		    	throw new
			    	IllegalArgumentException("The only parameter that can be " +
						     "used is --frame=_classname_");
		    }
		} else {
		    frameClassName = "org.nongnu.savannah.jcurzez.frame.SingleFrame";
		}
		
		Screen.getScreen().touch();
		Screen.getScreen().refresh();
	
		Thread turtle = new Thread(){
			public void run(){
				Area a = Screen.getScreen().constructChildArea(new Rectangle(5,5,30,20, new OrderedLock()), true, true);
				a.getRectangle().getFillingPen().setBackgroundColor(BackgroundColor.RED);
				a.getRectangle().getDrawingPen().setBackgroundColor(BackgroundColor.RED);
				a.stressConcurrency = true;
				a.clear();
				a.refresh();
				
				for (int i = 0 ; i < 5; i++){
					a.printString(i + ")  This is a long message that when printed in a window " +
							"that is artificially stressing concurrency will cause some lock to be held " +
							"for a long time.  Hopefully, other windows can proceed while the lock is held.\n\n");
					a.refresh();
					try{Thread.sleep(500);}catch(Exception e){}
				}
			}
		};
		
		turtle.start();
		
		Thread agile = new Thread(){
			public void run(){
				Area a2 =  Screen.getScreen().constructChildArea(new Rectangle(45,5,75,20, new OrderedLock()), true, true);
				a2.getRectangle().getFillingPen().setBackgroundColor(BackgroundColor.GREEN);
				a2.getRectangle().getDrawingPen().setBackgroundColor(BackgroundColor.GREEN);
				a2.stressConcurrency = false;
				a2.clear();
				a2.refresh();
				
				int i = 0;
				while (true){
					int y = Math.round((float) (Math.random() * 14.0));
					a2.getCursor().setXY(0, y);
					if (num % interval == 0) a2.printString("Fast message " + i + ".\n");
					i++;
					num++;
				}
			}
		};
		
		agile.setDaemon(true);
		
		agile.start();
		
		try{ turtle.join(); } catch (Exception ex){}
		
		System.out.println("Set cursor " + NumberFormat.getIntegerInstance().format(num) + " times.");
		System.out.println("    [printing message every " + interval + " cursor moves.]");
    }
}