/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.test;

import atomicsets.OrderedLock;
import org.nongnu.savannah.jcurzez.*;
import org.nongnu.savannah.jcurzez.frame.*;

/**
 * Tests the {@link jcurzez.Window Window} object.
 *
 * @author Thomas Girard
 * @version 1.0.0
 */
public class WindowMain {
    /**
     * Runs the {@link org.nongnu.savannah.jcurzez.test.Test} program using the
     * <code>Window</code> object.
     * <br>
     * One can specify the frame to use with
     * <code>--frame=<i>frameclassname</i></code>.
     */
    public static void main(String[] args) throws Exception {
	Frame frame;
	if (args.length > 0) {
	    if (args[0].startsWith("--frame=")) {
		String frameClassName = args[0].substring(8);
		frame =
		    (Frame) (Class.forName(frameClassName).newInstance());
	    } else {
		throw new
		    IllegalArgumentException("The only parameter that can be " +
					     "used is --frame=_classname_");
	    }
	} else {
	    frame = new SingleFrame();
	}

	Window win = /*new*/ Window.getWindow(new Rectangle(10, 10, 39, 20,
					      new Pen(ForegroundColor.YELLOW,
						      BackgroundColor.BLUE,
						      Attribute.BOLD),
					      new Pen(ForegroundColor.GREEN,
						      BackgroundColor.RED,
						      Attribute.NORMAL), new OrderedLock()),
				frame);
	Test t = new Test(win, 1500);
    }
}
