/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez;

import org.nongnu.savannah.jcurzez.frame.*;


/**
 * A <code>Window</code> has a mandatory {@link Frame}. It can be
 * {@link #move moved}.
 * <p>
 * <i>The size of the rectangle given to build a new window is be the inner
 * usable size.</i>
 *
 * @author Thomas Girard
 * @version 1.0.0
 *
 * @see jcurzez.frame.Frame
 */
public class Window extends Area {
//    /**
//     * Creates a new window, according to <code>rectangle</code>. It will
//     * be single framed, shown on the first {@link #refresh}, and will
//     * have autoscrolling and linewrapping capacities.
//     *
//     * @param     rectangle the bounds for this window.
//     *
//     * @see       jcurzez.frame.SingleFrame
//     */
//    public Window(Rectangle rectangle) {
//	this(rectangle, new SingleFrame());
//    }
//
//    /**
//     * Creates a new window, as specified by <code>rectangle</code> and
//     * <code>frame</code>. It will have autoscrolling and linewrapping
//     * capacities.
//     *
//     * @param     rectangle the bounds for this window.
//     * @param     frame the frame to use.
//     */
//    public Window(Rectangle rectangle, Frame frame) {
//	this(rectangle, true, true, frame);
//    }
//
//    /**
//     * Creates a new window, as specified by <code>rectangle</code>.
//     * It will have autoscrolling if <code>scroll</code>
//     * is <code>true</code>, and linewrapping if <code>linewrap</code> is
//     * <code>true</code>. The frame used will be
//     * {@link jcurzez.frame.SingleFrame}.
//     *
//     * @param     rectangle the bounds for this window.
//     * @param     linewrap <code>true</code> for linewrapping capacity.
//     * @param     scroll <code>true</code> for scrolling capacity.
//     */
//    public Window(Rectangle rectangle,
//		  boolean linewrap,
//		  boolean scroll) {
//	this(rectangle, linewrap, scroll, new SingleFrame());
//    }
//
//
//    /**
//     * Creates a new window, as specified by <code>rectangle</code> and
//     * <code>parent</code>. It will have autoscrolling and linewrapping
//     * capacities. The frame used will be {@link jcurzez.frame.SingleFrame}.
//     *
//     * @param     parent the parent of this window.
//     * @param     rectangle the bounds for this window.
//     */
//    public Window(AbstractWindow parent, Rectangle rectangle) {
//	this(parent, rectangle, true, true, new SingleFrame());
//    }
//
//
//    /**
//     * Creates a new window, as specified by <code>rectangle</code> and
//     * <code>frame</code>. It will have autoscrolling if <code>scroll</code>
//     * is <code>true</code>, and linewrapping if <code>linewrap</code> is
//     * <code>true</code>.
//     *
//     * @param     rectangle the bounds for this window.
//     * @param     linewrap <code>true</code> for linewrapping capacity.
//     * @param     scroll <code>true</code> for scrolling capacity.
//     * @param     frame the frame to use.
//     */
//    public Window(Rectangle rectangle,
//		  boolean linewrap,
//		  boolean scroll,
//		  Frame frame) {
//	this(Screen.getScreen(), rectangle, linewrap, scroll, frame);
//    }
//
//
//    /**
//     * Creates a new window, as specified by <code>rectangle</code> and
//     * <code>frame</code>. It will have autoscrolling if <code>scroll</code>
//     * is <code>true</code>, and linewrapping if <code>linewrap</code> is
//     * <code>true</code>. The parent will be <code>parentWindow</code>.
//     *
//     * @param     parentWindow the parent window.
//     * @param     rectangle the bounds for this window.
//     * @param     linewrap <code>true</code> for linewrapping capacity.
//     * @param     scroll <code>true</code> for scrolling capacity.
//     * @param     frame the frame to use.
//     */
//    public Window(AbstractWindow parentWindow,
//		  Rectangle rectangle,
//		  boolean linewrap,
//		  boolean scroll,
//		  Frame frame) {
//	super(new Parent(parentWindow, rectangle, frame),
//	      rectangle,
//	      linewrap,
//	      scroll);
//    }

	
	protected Window(Parent p, Rectangle r, boolean linewrap, boolean scroll){
		super(p,r,linewrap,scroll,true);
	}
	
	public static Window getWindow(AbstractWindow parentWindow, Rectangle rectangle,
			boolean linewrap, boolean scroll, Frame frame)
	{
		Parent p = new Parent(parentWindow, rectangle, frame);
		return p.constructChildWindow(rectangle, linewrap, scroll);
	}

	public static Window getWindow(Rectangle r, Frame f){
		return getWindow(Screen.getScreen(), r, true, true, f);
	}
	
	public static Window getWindow(Rectangle r){
		return getWindow(r, new SingleFrame());
	}
	
	public static Window getWindow(AbstractWindow parent, Rectangle r){
		return getWindow(parent, r, true, true, new SingleFrame());
	}


    /**
     * Since we've added a fake parent to <code>this</code>,
     * we hide it to the outside.
     */
    public AbstractWindow getParent() {
	return parent.getParent();
    }

    /**
     * Touches parent because the frame needs being touched.
     */
    public void touch() {
	parent.touch();
    }
    
    private boolean moved = false;
    
    @Override public void refresh(){
    	if (moved) {
    		moved = false;
    		parent.getParent().touch();
    		parent.getParent().refresh();
    	}
    	super.refresh();
    }

    /**
     * Changes the frame of the window.
     *
     * @param     aFrame the new frame.
     */
    public void setFrame(Frame aFrame) {
	Parent parentWindow = (Parent) parent;
	parentWindow.frame = aFrame;
	parentWindow.copyFrameToBuffer();
	touch();
    }

    
    public void move(int dx, int dy){
    	move(parent.cursor.rectangle, cursor.rectangle, dx, dy);
    }

    /**
     * Moves this window to specified offset.
     *
     * @param     dx x-offset.
     * @param     dy y-offset.
     * @exception InconsistentCoordinateException if the offset
     *            given makes this window be outside the parent's window.
     */
    public void move(/*unitfor(R)*/Rectangle pr, /*unitfor(R)*/Rectangle r, int dx, int dy) {
//		final AbstractWindow realParent = getParent();
//		final int maxWidth = realParent.cursor.rectangle.width;
//		final int maxHeight = realParent.cursor.rectangle.height;
//	
//		synchronized(parent.cursor.rectangle){			
//			synchronized(cursor.rectangle){
//				if ((parent.cursor.rectangle.getRight() + dx >= maxWidth) ||
//				    (parent.cursor.rectangle.left + dx < 0) ||
//				    (parent.cursor.rectangle.getBottom() + dy >= maxHeight) ||
//				    (parent.cursor.rectangle.top + dy < 0)) {
//				    throw new
//					InconsistentCoordinateException("New position would make " +
//									"part of the window fall " +
//									"outside its parent area");
//				}
//			
//				parent.cursor.rectangle.left += dx;
//				parent.cursor.rectangle.top += dy;
//			
//				cursor.rectangle.left += dx;
//				cursor.rectangle.top += dy;
//			}
//		}
    	
    	AbstractWindow realParent = getParent();
    	pr.move(dx, dy, realParent.cursor.rectangle);
    	r.move(dx, dy, pr);
		
		moved = true;
    }
    
    



}


/**
 * The parent we create, with the frame. This parent will create its
 * own buffer.
 */
/*private static*/ class Parent extends Area {

/** The frame associated to this window. It can be changed. */
 Frame frame;


/** Change the rectangle before invoking <code>super</code>. */
 Parent(AbstractWindow realParent,
	       Rectangle rectangle,
	       Frame frame) {
    super(realParent, createOuterRectangle(rectangle),
	  false, false, false);
    this.frame = frame;
    copyFrameToBuffer();
}

protected Window /*B=this.B*/ constructChildWindow(Rectangle r, boolean linewrap, boolean scroll){
	return new Window/*B=this.B*/(this, r, linewrap, scroll);
}

/**
 * Writes the frame to the buffer.
 */
 void copyFrameToBuffer() {
    int fW = cursor.rectangle.getWidth();
    int fH = cursor.rectangle.getHeight();

    int where[] = { cursor.x, cursor.y };

    cursor.setXY(0,0);
    printChar(frame.getUpperLeftCorner());
    for (int i = 0; i < fW - 2; i++) {
	printChar(frame.getHorizontalTopLine(i));
    }
    printChar(frame.getUpperRightCorner());

    for (int j = 0; j < fH - 2; j++) {
	cursor.setXY(0, j + 1);
	printChar(frame.getVerticalLeftLine(j));
	cursor.setXY(fW - 1, j + 1);
	printChar(frame.getVerticalRightLine(j));
    }

    cursor.setXY(0, fH - 1);
    printChar(frame.getLowerLeftCorner());
    for (int i = 0; i < fW - 2; i++) {
	printChar(frame.getHorizontalBottomLine(i));
    }
    printChar(frame.getLowerRightCorner());
    cursor.setXY(where[0], where[1]);
}


/**
 * Creates the outer rectangle. It will have the size of
 * <code>rectangle</code> plus the size of the frame.
 */
 static Rectangle createOuterRectangle(Rectangle rectangle) {
    return new Rectangle(rectangle.left - 1,
			 rectangle.top - 1,
			 rectangle.getRight() + 1,
			 rectangle.getBottom() + 1,
			 rectangle.fillingPen,
			 rectangle.fillingPen);
}
}

