/*  Jcurzez - A free Java implementation of curses
 *  Copyright (C) 2001, 2006  Thomas G. Girard <thomas.g.girard@free.fr>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.nongnu.savannah.jcurzez.frame;


/**
 * This interface is used by {@link jcurzez.Window} to display a frame.
 * A frame is displayed from left to right, and top to bottom, starting
 * from the upper left corner. A frame has no idea where it is located,
 * but it knows which part of itself it is required to return.
 *
 * @author Thomas Girard
 * @version 1.0.0
 *
 * @see SingleFrame
 */
public interface Frame {
    /**
     * Returns the upper left corner of the frame.
     *
     * @return    the upper left corner of the frame.
     */
    public char getUpperLeftCorner();

    /**
     * Returns the upper right corner of the frame.
     *
     * @return    the upper right corner of the frame.
     */
    public char getUpperRightCorner();

    /**
     * Returns the lower left corner of the frame.
     *
     * @return    the lower left corner of the frame.
     */
    public char getLowerLeftCorner();

    /**
     * Returns the lower right corner of the frame.
     *
     * @return    the lower right corner of the frame.
     */
    public char getLowerRightCorner();

    /**
     * Returns the vertical line to display on the left part of the window.
     *
     * @param     which indicates will be in the range <code>0</code>
     *            to <code>window.getRectangle().getWidth() - 1</code>.
     */
    public char getVerticalLeftLine(int which);

    /**
     * Returns the vertical line to display on the right part of the window.
     *
     * @param     which indicates will be in the range <code>0</code>
     *            to <code>window.getRectangle().getWidth() - 1</code>.
     */
    public char getVerticalRightLine(int which);

    /**
     * Returns the horizontal line to display on the top part of the window.
     *
     * @param     which indicates will be in the range <code>0</code>
     *            to <code>window.getRectangle().getHeight() - 1</code>.
     */
    public char getHorizontalTopLine(int which);

    /**
     * Returns the horizontal line to display on the bottom part of the window.
     *
     * @param     which indicates will be in the range <code>0</code>
     *            to <code>window.getRectangle().getHeight() - 1</code>.
     */
    public char getHorizontalBottomLine(int which);
}
